#ifndef __G66WSBAPI_H__
#define __G66WSBAPI_H__

#ifdef WIN32_LEAN_AND_MEAN
#include <basetyps.h>
#endif //WIN32_LEAN_AND_MEAN

#include <windows.h>

#pragma pack(push,1)

#define G66WSB_CLASS_ID_DEVICE_ENUMERATOR	            0x0066F901
#define G66WSB_CLASS_ID_DEVICE		                    0x0066F902

typedef BOOL (__stdcall *G66WSB_CREATE_INSTANCE)(UINT32 ClassId,VOID **Interface);

#define G66WSB_OPEN_DEMO                                ((char*)0)
#define G66WSB_OPEN_FIRST                               ((char*)-1)

#define G66WSB_INTERFACE_TYPE_PCIE                      0x00
#define G66WSB_INTERFACE_TYPE_USB2                      0x01
#define G66WSB_INTERFACE_TYPE_USB3                      0x02
#define G66WSB_INTERFACE_TYPE_DEMO                      0xFF

#define G66WSB_FRONT_PANEL_LED_MODE_OFF                 0
#define G66WSB_FRONT_PANEL_LED_MODE_ON                  1
#define G66WSB_FRONT_PANEL_LED_MODE_DIAG                2

#define G66WSB_DEVICE_STATE_HIGH_TEMPERATURE            0x00000001

#define G66WSB_TEMPERATURE_UNKNOWN                      -16384

#define G66WSB_DDC_SAMPLE_RATE                          312500
#define G66WSB_DDC_BANDWIDTH                            250000
#define G66WSB_AUDIO_SAMPLE_RATE                        64000

#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_0            0
#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_1            1
#define G66WSB_AUDIO_STREAM_CALLBACK_STAGE_2            2

typedef struct
{
    char        DevicePath[MAX_PATH];
    BYTE        InterfaceType;
    char        SerialNumber[9];
    WORD        HWVersion;    
    WORD        FWVersion[3];
    BYTE        EEPROMVersion;                   
    UINT32      MaxDDCCount;    
    UINT32      Flags;
} G66WSB_DEVICE_INFO;

typedef struct
{
    UINT32      Flags;
    INT32       Temperatures[3];
    UINT32      FanRPM;
    UINT64      DataTransferred;
    UINT64      DataLost;
} G66WSB_DEVICE_STATE;

#undef INTERFACE
#define INTERFACE IG66WSBDeviceEnumerator

DECLARE_INTERFACE(IG66WSBDeviceEnumerator)
{
    STDMETHOD_(void,Free)(THIS) PURE;

    STDMETHOD_(BOOL,Enumerate)(THIS) PURE;
    STDMETHOD_(UINT32,GetCount)(THIS) PURE;
    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ UINT32 DeviceIndex,G66WSB_DEVICE_INFO *DeviceInfo) PURE;
};

struct IG66WSBDevice;

#undef INTERFACE
#define INTERFACE IG66WSBDeviceCallback

DECLARE_INTERFACE(IG66WSBDeviceCallback)
{    
    STDMETHOD_(void,G66WSB_DDCStreamCallback)(THIS_ struct IG66WSBDevice *Device,UINT32 DDCId,const float *Buffer,UINT32 Count,float Peak,float RMS) PURE;
    STDMETHOD_(void,G66WSB_AudioStreamCallback)(THIS_ struct IG66WSBDevice *Device,UINT32 DDCId,UINT32 Stage,const float *Buffer,UINT32 Count) PURE;    
    STDMETHOD_(void,G66WSB_ChannelActivityCallback)(THIS_ struct IG66WSBDevice *Device,const float *RMS,const float *dBm,WORD ADCLevel) PURE;
};

#define G66WSB_DEVICE_INTERFACE_VERSION         0x00000001

#undef INTERFACE
#define INTERFACE IG66WSBDevice

DECLARE_INTERFACE(IG66WSBDevice)
{    
    STDMETHOD_(BOOL,Free)(THIS) PURE;
    STDMETHOD_(BOOL,GetInterfaceVersion)(THIS_ UINT32 *Version) PURE;    

    STDMETHOD_(void,SetCallback)(THIS_ IG66WSBDeviceCallback *Callback) PURE;
    STDMETHOD_(IG66WSBDeviceCallback*,GetCallback)(THIS) PURE;

    STDMETHOD_(BOOL,Open)(THIS_ const char *Path) PURE;
    STDMETHOD_(BOOL,Close)(THIS) PURE;

    STDMETHOD_(BOOL,IsOpen)(THIS) PURE;

    STDMETHOD_(BOOL,GetDeviceInfo)(THIS_ G66WSB_DEVICE_INFO *DeviceInfo) PURE;
    
    STDMETHOD_(BOOL,IsConnected)(THIS_ BOOL *Connected) PURE;

    STDMETHOD_(BOOL,SetLED)(THIS_ UINT32 LEDMode) PURE;
    STDMETHOD_(BOOL,GetLED)(THIS_ UINT32 *LEDMode) PURE;

    STDMETHOD_(BOOL,SetPower)(THIS_ BOOL Power) PURE;
    STDMETHOD_(BOOL,GetPower)(THIS_ BOOL *Power) PURE;    
    
    STDMETHOD_(BOOL,GetDeviceState)(THIS_ G66WSB_DEVICE_STATE *State) PURE;
    
    STDMETHOD_(BOOL,SetBuiltInTest)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetBuiltInTest)(THIS_ BOOL *Enabled) PURE;

    STDMETHOD_(BOOL,SetAttenuator)(THIS_ UINT32 Attenuator) PURE;
    STDMETHOD_(BOOL,GetAttenuator)(THIS_ UINT32 *Attenuator) PURE;

    STDMETHOD_(BOOL,SetPreamplifier)(THIS_ BOOL Preamp) PURE;
    STDMETHOD_(BOOL,GetPreamplifier)(THIS_ BOOL *Preamp) PURE;    

    STDMETHOD_(BOOL,SetADCNoiseBlanker)(THIS_ BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlanker)(THIS_ BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetADCNoiseBlankerThreshold)(THIS_ WORD Threshold) PURE;
    STDMETHOD_(BOOL,GetADCNoiseBlankerThreshold)(THIS_ WORD *Threshold) PURE;
    
    STDMETHOD_(BOOL,CreateDDC)(THIS_ UINT32 *DDCId) PURE;    
    STDMETHOD_(BOOL,DeleteDDC)(THIS_ UINT32 DDCId) PURE;
        
    STDMETHOD_(BOOL,StartDDC)(THIS_ UINT32 DDCId,UINT32 SampleSetsPerBuffer) PURE;
    STDMETHOD_(BOOL,StopDDC)(THIS_ UINT32 DDCId) PURE;
    
    STDMETHOD_(BOOL,SetChannel)(THIS_ UINT32 DDCId,UINT32 Channel) PURE;
    STDMETHOD_(BOOL,GetChannel)(THIS_ UINT32 DDCId,UINT32 *Channel,UINT32 *Frequency) PURE;

    STDMETHOD_(BOOL,GetChannelFrequency)(THIS_ UINT32 Channel,UINT32 *Frequency) PURE;
    
    STDMETHOD_(BOOL,GetSignalLevel)(THIS_ UINT32 DDCId,float *Peak,float *RMS) PURE;
        
    STDMETHOD_(BOOL,StartAudio)(THIS_ UINT32 DDCId,UINT32 SamplesPerBuffer) PURE;
    STDMETHOD_(BOOL,StopAudio)(THIS_ UINT32 DDCId) PURE;
    
    STDMETHOD_(BOOL,SetAudioFilter)(THIS_ UINT32 DDCId, BOOL Enabled) PURE;
    STDMETHOD_(BOOL,GetAudioFilter)(THIS_ UINT32 DDCId, BOOL *Enabled) PURE;
    STDMETHOD_(BOOL,SetAudioFilterParams)(THIS_ UINT32 DDCId, UINT32 CutOffLow, UINT32 CutOffHigh, double Deemphasis) PURE;
    STDMETHOD_(BOOL,GetAudioFilterParams)(THIS_ UINT32 DDCId, UINT32 *CutOffLow, UINT32 *CutOffHigh, double *Deemphasis) PURE;
    STDMETHOD_(BOOL,SetAudioFilterLength)(THIS_ UINT32 DDCId, UINT32 Length) PURE;
    STDMETHOD_(BOOL,GetAudioFilterLength)(THIS_ UINT32 DDCId, UINT32 *Length) PURE;
    STDMETHOD_(BOOL,SetAudioGain)(THIS_ UINT32 DDCId, double Gain) PURE;
    STDMETHOD_(BOOL,GetAudioGain)(THIS_ UINT32 DDCId, double *Gain) PURE;
    STDMETHOD_(BOOL,SetVolume)(THIS_ UINT32 DDCId, BYTE Volume) PURE;
    STDMETHOD_(BOOL,GetVolume)(THIS_ UINT32 DDCId, BYTE *Volume) PURE;
    STDMETHOD_(BOOL,SetMute)(THIS_ UINT32 DDCId, BOOL Mute) PURE;
    STDMETHOD_(BOOL,GetMute)(THIS_ UINT32 DDCId, BOOL *Mute) PURE;

    STDMETHOD_(BOOL,SetBandwidth)(THIS_ UINT32 Bandwidth) PURE;
    STDMETHOD_(BOOL,GetBandwidth)(THIS_ UINT32 *Bandwidth) PURE;

    STDMETHOD_(BOOL,StartChannelActivity)(THIS_ WORD Interval) PURE;
    STDMETHOD_(BOOL,StopChannelActivity)(THIS) PURE;

    STDMETHOD_(BOOL,GetSpectrumCompensation)(THIS_ UINT32 Channel,float *Value) PURE;
    
    STDMETHOD_(BOOL,GetVersion)(THIS_ UINT64 *Version) PURE;    
};

  
typedef void (__stdcall *G66WSB_DDC_STREAM_CALLBACK)(UINT32 DDCId,const float *Buffer,UINT32 Count,float Peak,float RMS,DWORD_PTR UserData);
typedef void (__stdcall *G66WSB_AUDIO_STREAM_CALLBACK)(UINT32 DDCId,UINT32 Stage,const float *Buffer,UINT32 Count,DWORD_PTR UserData);
typedef void (__stdcall *G66WSB_CHANNEL_ACTIVITY_CALLBACK)(const float *RMS,const float *dBm,WORD ADCLevel,DWORD_PTR UserData);

typedef struct
{    
    G66WSB_DDC_STREAM_CALLBACK          DDCStreamCallback;    
    G66WSB_AUDIO_STREAM_CALLBACK        AudioStreamCallback;    
    G66WSB_CHANNEL_ACTIVITY_CALLBACK    ChannelActivityCallback;
} G66WSB_CALLBACKS;

#pragma pack(pop)

typedef BOOL (__stdcall *G66WSB_GET_DEVICE_LIST)(G66WSB_DEVICE_INFO **DeviceList,UINT32 *Count);
typedef void (__stdcall *G66WSB_FREE_DEVICE_LIST)(G66WSB_DEVICE_INFO *DeviceList);
typedef INT32 (__stdcall *G66WSB_OPEN_DEVICE)(const char *SerialNumber);
typedef BOOL (__stdcall *G66WSB_CLOSE_DEVICE)(INT32 hDevice);
typedef BOOL (__stdcall *G66WSB_IS_DEVICE_CONNECTED)(INT32 hDevice,BOOL *Connected);
typedef BOOL (__stdcall *G66WSB_SET_CALLBACKS)(INT32 hDevice,const G66WSB_CALLBACKS *Callbacks,DWORD_PTR UserData);
typedef BOOL (__stdcall *G66WSB_GET_DEVICE_INFO)(INT32 hDevice,G66WSB_DEVICE_INFO *DeviceInfo);
typedef BOOL (__stdcall *G66WSB_SET_LED)(INT32 hDevice,UINT32 LEDMode);
typedef BOOL (__stdcall *G66WSB_GET_LED)(INT32 hDevice,UINT32 *LEDMode);
typedef BOOL (__stdcall *G66WSB_SET_POWER)(INT32 hDevice,BOOL Power);
typedef BOOL (__stdcall *G66WSB_GET_POWER)(INT32 hDevice,BOOL *Power);
typedef BOOL (__stdcall *G66WSB_GET_DEVICE_STATE)(INT32 hDevice,G66WSB_DEVICE_STATE *State);
typedef BOOL (__stdcall *G66WSB_SET_BUILT_IN_TEST)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G66WSB_GET_BUILT_IN_TEST)(INT32 hDevice,BOOL *Enabled);    
typedef BOOL (__stdcall *G66WSB_SET_ATTENUATOR)(INT32 hDevice,UINT32 Attenuator);
typedef BOOL (__stdcall *G66WSB_GET_ATTENUATOR)(INT32 hDevice,UINT32 *Attenuator);
typedef BOOL (__stdcall *G66WSB_SET_PREAMPLIFIER)(INT32 hDevice,BOOL Preamp);
typedef BOOL (__stdcall *G66WSB_GET_PREAMPLIFIER)(INT32 hDevice,BOOL *Preamp);
typedef BOOL (__stdcall *G66WSB_SET_ADC_NOISE_BLANKER)(INT32 hDevice,BOOL Enabled);
typedef BOOL (__stdcall *G66WSB_GET_ADC_NOISE_BLANKER)(INT32 hDevice,BOOL *Enabled);
typedef BOOL (__stdcall *G66WSB_SET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,WORD Threshold);
typedef BOOL (__stdcall *G66WSB_GET_ADC_NOISE_BLANKER_THRESHOLD)(INT32 hDevice,WORD *Threshold);
typedef BOOL (__stdcall *G66WSB_CREATE_DDC)(INT32 hDevice,UINT32 *DDCId);
typedef BOOL (__stdcall *G66WSB_DELETE_DDC)(INT32 hDevice,UINT32 DDCId);
typedef BOOL (__stdcall *G66WSB_START_DDC)(INT32 hDevice,UINT32 DDCId,UINT32 SampleSetsPerBuffer);
typedef BOOL (__stdcall *G66WSB_STOP_DDC)(INT32 hDevice,UINT32 DDCId);
typedef BOOL (__stdcall *G66WSB_SET_CHANNEL)(INT32 hDevice,UINT32 DDCId,UINT32 Channel);
typedef BOOL (__stdcall *G66WSB_GET_CHANNEL)(INT32 hDevice,UINT32 DDCId,UINT32 *Channel,UINT32 *Frequency);
typedef BOOL (__stdcall *G66WSB_GET_SIGNAL_LEVEL)(INT32 hDevice,UINT32 DDCId,float *Peak,float *RMS);
typedef BOOL (__stdcall *G66WSB_START_AUDIO)(INT32 hDevice,UINT32 DDCId,UINT32 SamplesPerBuffer);
typedef BOOL (__stdcall *G66WSB_STOP_AUDIO)(INT32 hDevice,UINT32 DDC2ChannelId);
typedef BOOL (__stdcall *G66WSB_SET_AUDIO_FILTER)(INT32 hDevice,UINT32 DDCId,BOOL Enabled);
typedef BOOL (__stdcall *G66WSB_GET_AUDIO_FILTER)(INT32 hDevice,UINT32 DDCId,BOOL *Enabled);
typedef BOOL (__stdcall *G66WSB_SET_AUDIO_FILTER_PARAMS)(INT32 hDevice,UINT32 DDCId,UINT32 CutOffLow,UINT32 CutOffHigh,double Deemphasis); 
typedef BOOL (__stdcall *G66WSB_GET_AUDIO_FILTER_PARAMS)(INT32 hDevice,UINT32 DDCId,UINT32 *CutOffLow,UINT32 *CutOffHigh,double *Deemphasis); 
typedef BOOL (__stdcall *G66WSB_SET_AUDIO_FILTER_LENGTH)(INT32 hDevice,UINT32 DDCId,UINT32 Length);
typedef BOOL (__stdcall *G66WSB_GET_AUDIO_FILTER_LENGTH)(INT32 hDevice,UINT32 DDCId,UINT32 *Length);
typedef BOOL (__stdcall *G66WSB_SET_AUDIO_GAIN)(INT32 hDevice,UINT32 DDCId,double Gain);
typedef BOOL (__stdcall *G66WSB_GET_AUDIO_GAIN)(INT32 hDevice,UINT32 DDCId,double *Gain);
typedef BOOL (__stdcall *G66WSB_SET_VOLUME)(INT32 hDevice,UINT32 DDCId,BYTE Volume);
typedef BOOL (__stdcall *G66WSB_GET_VOLUME)(INT32 hDevice,UINT32 DDCId,BYTE *Volume);
typedef BOOL (__stdcall *G66WSB_SET_MUTE)(INT32 hDevice,UINT32 DDCId,BOOL Mute);
typedef BOOL (__stdcall *G66WSB_GET_MUTE)(INT32 hDevice,UINT32 DDCId,BOOL *Mute);
typedef BOOL (__stdcall *G66WSB_GET_SPECTRUM_COMPENSATION)(INT32 hDevice,UINT32 Channel,float *Value);
typedef BOOL (__stdcall *G66WSB_SET_BANDWIDTH)(INT32 hDevice,UINT32 Bandwidth);
typedef BOOL (__stdcall *G66WSB_GET_BANDWIDTH)(INT32 hDevice,UINT32 *Bandwidth);
typedef BOOL (__stdcall *G66WSB_START_CHANNEL_ACTIVITY)(INT32 hDevice,WORD Interval);
typedef BOOL (__stdcall *G66WSB_STOP_CHANNEL_ACTIVITY)(INT32 hDevice);
typedef BOOL (__stdcall *G66WSB_GET_CHANNEL_FREQUENCY)(INT32 hDevice,UINT32 Channel,UINT32 *Frequency);


#endif //__G66WSBAPI_H__

